@@ -17,25 +17,34 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-type t = {
-  common : CommonArg.t;
-  (* compile option *)
-  profile : Driver.profile option;
-  source_map : (string * Source_map.t) option;
-  runtime_files : string list;
-  output_file : string option;
-  input_file : string option;
-  params : (string * string) list;
-  (* toplevel *)
-  linkall : bool;
-  toplevel : bool;
-  nocmis : bool;
-  (* filesystem *)
-  include_dir : string list;
-  fs_files : string list;
-  fs_output : string option;
-  fs_external : bool;
-}
+open Js_of_ocaml_compiler
+
+type t =
+  { common : CommonArg.t
+  ; (* compile option *)
+    profile : Driver.profile option
+  ; source_map : (string option * Source_map.t) option
+  ; runtime_files : string list
+  ; runtime_only : bool
+  ; output_file : [ `Name of string | `Stdout ] * bool
+  ; input_file : string option
+  ; params : (string * string) list
+  ; static_env : (string * string) list
+  ; wrap_with_fun : string option
+  ; (* toplevel *)
+    dynlink : bool
+  ; linkall : bool
+  ; toplevel : bool
+  ; export_file : string option
+  ; nocmis : bool
+  ; (* filesystem *)
+    include_dir : string list
+  ; fs_files : string list
+  ; fs_output : string option
+  ; fs_external : bool
+  ; keep_unit_names : bool
+  }
 
 val options : t Cmdliner.Term.t
+
 val info : Cmdliner.Term.info
