@@ -229,7 +229,7 @@ function caml_int64_div (x, y)
 function caml_int64_mod (x, y)
 {
   if (caml_int64_is_zero (y)) caml_raise_zero_divide ();
-  var sign = x[3] ^ y[3];
+  var sign = x[3];
   if (x[3] & 0x8000) x = caml_int64_neg(x);
   if (y[3] & 0x8000) y = caml_int64_neg(y);
   var r = caml_int64_udivmod(x, y)[2];
@@ -289,7 +289,7 @@ function caml_int64_format (fmt, x) {
 }
 
 //Provides: caml_int64_of_string
-//Requires: caml_parse_sign_and_base, caml_failwith, caml_parse_digit, MlString
+//Requires: caml_parse_sign_and_base, caml_failwith, caml_parse_digit, MlBytes
 //Requires: caml_int64_of_int32, caml_int64_udivmod, caml_int64_ult
 //Requires: caml_int64_add, caml_int64_mul, caml_int64_neg
 //Requires: caml_ml_string_length,caml_string_unsafe_get
@@ -298,7 +298,7 @@ function caml_int64_of_string(s) {
   var i = r[0], sign = r[1], base = r[2];
   var base64 = caml_int64_of_int32(base);
   var threshold =
-    caml_int64_udivmod([255, 0xffffff, 0xfffffff, 0xffff], base64)[1];
+      caml_int64_udivmod([255, 0xffffff, 0xfffffff, 0xffff], base64)[1];
   var c = caml_string_unsafe_get(s, i);
   var d = caml_parse_digit(c);
   if (d < 0 || d >= base) caml_failwith("int_of_string");
@@ -323,6 +323,27 @@ function caml_int64_of_string(s) {
   return res;
 }
 
+//Provides: caml_int64_create_lo_mi_hi
+function caml_int64_create_lo_mi_hi(lo, mi, hi){
+  return [255, lo, mi, hi]
+}
+//Provides: caml_int64_create_lo_hi
+function caml_int64_create_lo_hi(lo, hi){
+  return [255,
+          lo & 0xffffff,
+          ((lo >>> 24) & 0xff) | ((hi & 0xffff) << 8),
+          (hi >>> 16) & 0xffff];
+}
+//Provides: caml_int64_lo32 const
+function caml_int64_lo32(v){
+  return v[1] | ((v[2] & 0xff) << 24);
+}
+//Provides: caml_int64_hi32 const
+function caml_int64_hi32(v){
+  return ((v[2] >>> 8) & 0xffff) | (v[3] << 16);
+}
+
+
 //Provides: caml_int64_of_bytes
 function caml_int64_of_bytes(a) {
   return [255, a[7] | (a[6] << 8) | (a[5] << 16),
@@ -333,3 +354,10 @@ function caml_int64_to_bytes(x) {
   return [x[3] >> 8, x[3] & 0xff, x[2] >> 16, (x[2] >> 8) & 0xff, x[2] & 0xff,
           x[1] >> 16, (x[1] >> 8) & 0xff, x[1] & 0xff];
 }
+
+//Provides: caml_int64_hash
+function caml_int64_hash(x){
+  var lo = ((x[1]      )         ) | ((x[2] & 0xff) << 24);
+  var hi = ((x[2] >>> 8) & 0xffff) | ((x[3]       ) << 16);
+  return lo ^ hi;
+}
