@@ -22,42 +22,50 @@
 //Provides: caml_js_pure_expr const
 function caml_js_pure_expr (f) { return f(); }
 
-//Provides: caml_js_set
+//Provides: caml_js_set (mutable, const, const)
 function caml_js_set(o,f,v) { o[f]=v;return 0}
-//Provides: caml_js_get mutable
+//Provides: caml_js_get mutable (const, const)
 function caml_js_get(o,f) { return o[f]; }
-//Provides: caml_js_delete
+//Provides: caml_js_delete (mutable, const)
 function caml_js_delete(o,f) { delete o[f]; return 0}
 
-//Provides: caml_js_instanceof
+//Provides: caml_js_instanceof (const, const)
 function caml_js_instanceof(o,c) { return o instanceof c; }
 
-//Provides: caml_js_typeof
+//Provides: caml_js_typeof (const)
 function caml_js_typeof(o) { return typeof o; }
 
 //Provides: caml_js_on_ie const
 function caml_js_on_ie () {
   var ua =
-    joo_global_object.navigator?joo_global_object.navigator.userAgent:"";
+      joo_global_object.navigator?joo_global_object.navigator.userAgent:"";
   return ua.indexOf("MSIE") != -1 && ua.indexOf("Opera") != 0;
 }
 
-//Provides: caml_js_html_escape const
+//Provides: caml_js_html_escape const (const)
 var caml_js_regexps = { amp:/&/g, lt:/</g, quot:/\"/g, all:/[&<\"]/ };
 function caml_js_html_escape (s) {
   if (!caml_js_regexps.all.test(s)) return s;
   return s.replace(caml_js_regexps.amp, "&amp;")
-          .replace(caml_js_regexps.lt, "&lt;")
-          .replace(caml_js_regexps.quot, "&quot;");
+    .replace(caml_js_regexps.lt, "&lt;")
+    .replace(caml_js_regexps.quot, "&quot;");
 }
 
 //Provides: caml_js_html_entities
+//Requires: caml_failwith
 function caml_js_html_entities(s) {
+  var entity = /^&#?[0-9a-zA-Z]+;$/
+  if(s.match(entity))
+  {
     var str, temp = document.createElement('p');
     temp.innerHTML= s;
     str= temp.textContent || temp.innerText;
     temp=null;
     return str;
+  }
+  else {
+    caml_failwith("Invalid entity " + s);
+  }
 }
 
 /////////// Debugging console
@@ -78,7 +86,6 @@ function caml_trampoline(res) {
     res = res.joo_tramp.apply(null, res.joo_args);
     c++;
   }
-  //if(c>10) joo_global_object.console.log("trampoline ", c, "times")
   return res;
 }
 
@@ -87,21 +94,35 @@ function caml_trampoline_return(f,args) {
   return {joo_tramp:f,joo_args:args};
 }
 
-//Provides: js_print_stdout
+//Provides: js_print_stdout (const)
+//Requires: caml_utf16_of_utf8
 function js_print_stdout(s) {
-  // Do not output the last \n if present
-  // as console logging display a newline at the end
-  if(s.charCodeAt(s.length - 1) == 10)
-    s = s.substr(0,s.length - 1 );
-  var v = joo_global_object.console;
-  v  && v.log && v.log(s);
+  var s = caml_utf16_of_utf8(s);
+  var g = joo_global_object;
+  if (g.process && g.process.stdout && g.process.stdout.write) {
+    g.process.stdout.write(s)
+  } else {
+    // Do not output the last \n if present
+    // as console logging display a newline at the end
+    if(s.charCodeAt(s.length - 1) == 10)
+      s = s.substr(0,s.length - 1 );
+    var v = g.console;
+    v  && v.log && v.log(s);
+  }
 }
-//Provides: js_print_stderr
+//Provides: js_print_stderr (const)
+//Requires: caml_utf16_of_utf8
 function js_print_stderr(s) {
-  // Do not output the last \n if present
-  // as console logging display a newline at the end
-  if(s.charCodeAt(s.length - 1) == 10)
-    s = s.substr(0,s.length - 1 );
-  var v = joo_global_object.console;
-  v && v.error && v.error(s);
+  var s = caml_utf16_of_utf8(s);
+  var g = joo_global_object;
+  if (g.process && g.process.stdout && g.process.stdout.write) {
+    g.process.stderr.write(s)
+  } else {
+    // Do not output the last \n if present
+    // as console logging display a newline at the end
+    if(s.charCodeAt(s.length - 1) == 10)
+      s = s.substr(0,s.length - 1 );
+    var v = g.console;
+    v && v.error && v.error(s);
+  }
 }
