@@ -41,14 +41,16 @@ var caml_marshal_constants = {
   CODE_DOUBLE_ARRAY32_LITTLE: 0x07,
   CODE_CODEPOINTER:           0x10,
   CODE_INFIXPOINTER:          0x11,
-  CODE_CUSTOM:                0x12
+  CODE_CUSTOM:                0x12,
+  CODE_CUSTOM_LEN:            0x18,
+  CODE_CUSTOM_FIXED:          0x19
 }
 
 
-//Provides: MlStringReader
-//Requires: caml_new_string, caml_bytes_of_string
-function MlStringReader (s, i) { this.s = caml_bytes_of_string(s); this.i = i; }
-MlStringReader.prototype = {
+//Provides: MlBytesReader
+//Requires: caml_new_string, caml_jsbytes_of_string
+function MlBytesReader (s, i) { this.s = caml_jsbytes_of_string(s); this.i = i; }
+MlBytesReader.prototype = {
   read8u:function () { return this.s.charCodeAt(this.i++); },
   read8s:function () { return this.s.charCodeAt(this.i++) << 24 >> 24; },
   read16u:function () {
@@ -80,6 +82,47 @@ MlStringReader.prototype = {
   }
 }
 
+//Provides: BigStringReader
+//Requires: caml_string_of_array, caml_ba_get_1
+function BigStringReader (bs, i) { this.s = bs; this.i = i; }
+BigStringReader.prototype = {
+  read8u:function () { return caml_ba_get_1(this.s,this.i++); },
+  read8s:function () { return caml_ba_get_1(this.s,this.i++) << 24 >> 24; },
+  read16u:function () {
+    var s = this.s, i = this.i;
+    this.i = i + 2;
+    return (caml_ba_get_1(s,i) << 8) | caml_ba_get_1(s,i + 1)
+  },
+  read16s:function () {
+    var s = this.s, i = this.i;
+    this.i = i + 2;
+    return (caml_ba_get_1(s,i) << 24 >> 16) | caml_ba_get_1(s,i + 1);
+  },
+  read32u:function () {
+    var s = this.s, i = this.i;
+    this.i = i + 4;
+    return ((caml_ba_get_1(s,i)   << 24) | (caml_ba_get_1(s,i+1) << 16) |
+            (caml_ba_get_1(s,i+2) << 8)  | caml_ba_get_1(s,i+3)         ) >>> 0;
+  },
+  read32s:function () {
+    var s = this.s, i = this.i;
+    this.i = i + 4;
+    return (caml_ba_get_1(s,i)   << 24) | (caml_ba_get_1(s,i+1) << 16) |
+      (caml_ba_get_1(s,i+2) << 8)  | caml_ba_get_1(s,i+3);
+  },
+  readstr:function (len) {
+    var i = this.i;
+    var arr = new Array(len)
+    for(var j = 0; j < len; j++){
+      arr[j] = caml_ba_get_1(this.s, i+j);
+    }
+    this.i = i + len;
+    return caml_string_of_array(arr);
+  }
+}
+
+
+
 //Provides: caml_float_of_bytes
 //Requires: caml_int64_float_of_bits, caml_int64_of_bytes
 function caml_float_of_bytes (a) {
@@ -87,12 +130,89 @@ function caml_float_of_bytes (a) {
 }
 
 //Provides: caml_input_value_from_string mutable
+//Requires: MlBytesReader, caml_input_value_from_reader
+function caml_input_value_from_string(s,ofs) {
+  var reader = new MlBytesReader (s, typeof ofs=="number"?ofs:ofs[0]);
+  return caml_input_value_from_reader(reader, ofs)
+}
+
+//Provides: caml_input_value_from_bytes mutable
+//Requires: MlBytesReader, caml_input_value_from_reader
+function caml_input_value_from_bytes(s,ofs) {
+  var reader = new MlBytesReader (s, typeof ofs=="number"?ofs:ofs[0]);
+  return caml_input_value_from_reader(reader, ofs)
+}
+
+//Provides: caml_int64_unmarshal
+//Requires: caml_int64_of_bytes
+function caml_int64_unmarshal(reader, size){
+  var t = new Array(8);;
+  for (var j = 0;j < 8;j++) t[j] = reader.read8u();
+  size[0] = 8;
+  return caml_int64_of_bytes (t);
+}
+
+//Provides: caml_int64_marshal
+//Requires: caml_int64_to_bytes
+function caml_int64_marshal(writer, v, sizes) {
+  var b = caml_int64_to_bytes (v);
+  for (var i = 0; i < 8; i++) writer.write (8, b[i]);
+  sizes[0] = 8; sizes[1] = 8;
+}
+
+//Provides: caml_int32_unmarshal
+function caml_int32_unmarshal(reader, size){
+  size[0] = 4;
+  return reader.read32s ();
+}
+
+//Provides: caml_nativeint_unmarshal
 //Requires: caml_failwith
-//Requires: caml_float_of_bytes, caml_int64_of_bytes
-//Requires: MlStringReader
-function caml_input_value_from_string(s, ofs) {
-  var reader = new MlStringReader (s, typeof ofs=="number"?ofs:ofs[0]);
-  var _magic = reader.read32u ();
+function caml_nativeint_unmarshal(reader, size){
+  switch (reader.read8u ()) {
+  case 1:
+    size[0] = 4;
+    return reader.read32s ();
+  case 2:
+    caml_failwith("input_value: native integer value too large");
+  default: caml_failwith("input_value: ill-formed native integer");
+  }
+}
+
+//Provides: caml_custom_ops
+//Requires: caml_int64_unmarshal, caml_int64_marshal, caml_int64_compare, caml_int64_hash
+//Requires: caml_int32_unmarshal, caml_nativeint_unmarshal
+//Requires: caml_ba_serialize, caml_ba_deserialize, caml_ba_compare, caml_ba_hash
+var caml_custom_ops =
+    {"_j": {
+      deserialize : caml_int64_unmarshal,
+      serialize  : caml_int64_marshal,
+      fixed_length : 8,
+      compare : caml_int64_compare,
+      hash : caml_int64_hash
+    },
+     "_i": {
+       deserialize : caml_int32_unmarshal,
+       fixed_length : 4,
+     },
+     "_n": {
+       deserialize : caml_nativeint_unmarshal,
+       fixed_length : 4,
+     },
+     "_bigarray":{
+       deserialize : caml_ba_deserialize,
+       serialize : caml_ba_serialize,
+       compare : caml_ba_compare,
+       hash: caml_ba_hash,
+     }
+    }
+
+//Provides: caml_input_value_from_reader mutable
+//Requires: caml_failwith
+//Requires: caml_float_of_bytes, caml_custom_ops
+
+function caml_input_value_from_reader(reader, ofs) {
+  var _magic = reader.read32u ()
   var _block_len = reader.read32u ();
   var num_objects = reader.read32u ();
   var _size_32 = reader.read32u ();
@@ -131,14 +251,14 @@ function caml_input_value_from_string(s, ofs) {
           caml_failwith("input_value: integer too large");
           break;
         case 0x04: //cst.CODE_SHARED8:
-          var ofs = reader.read8u ();
-          return intern_obj_table[obj_counter - ofs];
+          var offset = reader.read8u ();
+          return intern_obj_table[obj_counter - offset];
         case 0x05: //cst.CODE_SHARED16:
-          var ofs = reader.read16u ();
-          return intern_obj_table[obj_counter - ofs];
+          var offset = reader.read16u ();
+          return intern_obj_table[obj_counter - offset];
         case 0x06: //cst.CODE_SHARED32:
-          var ofs = reader.read32u ();
-          return intern_obj_table[obj_counter - ofs];
+          var offset = reader.read32u ();
+          return intern_obj_table[obj_counter - offset];
         case 0x08: //cst.CODE_BLOCK32:
           var header = reader.read32u ();
           var tag = header & 0xFF;
@@ -221,36 +341,37 @@ function caml_input_value_from_string(s, ofs) {
           caml_failwith ("input_value: code pointer");
           break;
         case 0x12: //cst.CODE_CUSTOM:
+        case 0x18: //cst.CODE_CUSTOM_LEN:
+        case 0x19: //cst.CODE_CUSTOM_FIXED:
           var c, s = "";
           while ((c = reader.read8u ()) != 0) s += String.fromCharCode (c);
-          switch(s) {
-          case "_j":
-            // Int64
-            var t = new Array(8);;
-            for (var j = 0;j < 8;j++) t[j] = reader.read8u();
-            var v = caml_int64_of_bytes (t);
-            if (intern_obj_table) intern_obj_table[obj_counter++] = v;
-            return v;
-          case "_i":
-            // Int32
-            var v = reader.read32s ();
-            if (intern_obj_table) intern_obj_table[obj_counter++] = v;
-            return v;
-          case "_n":
-            // Nativeint
-            switch (reader.read8u ()) {
-            case 1:
-              var v = reader.read32s ();
-              if (intern_obj_table) intern_obj_table[obj_counter++] = v;
-              return v;
-            case 2:
-              caml_failwith("input_value: native integer value too large");
-            default:
-              caml_failwith("input_value: ill-formed native integer");
-            }
-          default:
+          var ops = caml_custom_ops[s];
+          var expected_size;
+          if(!ops)
             caml_failwith("input_value: unknown custom block identifier");
+          switch(code){
+          case 0x12: // cst.CODE_CUSTOM (deprecated)
+            break;
+          case 0x19: // cst.CODE_CUSTOM_FIXED
+            if(!ops.fixed_length)
+              caml_failwith("input_value: expected a fixed-size custom block");
+            expected_size = ops.fixed_length;
+            break;
+          case 0x18: // cst.CODE_CUSTOM_LEN
+            expected_size = reader.read32u ();
+            // Skip size64
+            reader.read32s(); reader.read32s();
+            break;
           }
+          var old_pos = reader.i;
+          var size = [0];
+          var v = ops.deserialize(reader, size);
+          if(expected_size != undefined){
+            if(expected_size != size[0])
+              caml_failwith("input_value: incorrect length of serialized custom block");
+          }
+          if (intern_obj_table) intern_obj_table[obj_counter++] = v;
+          return v;
         default:
           caml_failwith ("input_value: ill-formed message");
         }
@@ -270,23 +391,70 @@ function caml_input_value_from_string(s, ofs) {
 }
 
 //Provides: caml_marshal_data_size mutable
-//Requires: caml_failwith, caml_string_unsafe_get
+//Requires: caml_failwith, caml_bytes_unsafe_get
 function caml_marshal_data_size (s, ofs) {
   function get32(s,i) {
-    return (caml_string_unsafe_get(s, i) << 24) |
-           (caml_string_unsafe_get(s, i + 1) << 16) |
-           (caml_string_unsafe_get(s, i + 2) << 8) |
-            caml_string_unsafe_get(s, i + 3);
+    return (caml_bytes_unsafe_get(s, i) << 24) |
+      (caml_bytes_unsafe_get(s, i + 1) << 16) |
+      (caml_bytes_unsafe_get(s, i + 2) << 8) |
+      caml_bytes_unsafe_get(s, i + 3);
   }
   if (get32(s, ofs) != (0x8495A6BE|0))
     caml_failwith("Marshal.data_size: bad object");
   return (get32(s, ofs + 4));
 }
 
+//Provides: MlObjectTable
+var MlObjectTable;
+if (typeof joo_global_object.WeakMap === 'undefined') {
+  MlObjectTable = function() {
+    /* polyfill (using linear search) */
+    function NaiveLookup(objs) { this.objs = objs; }
+    NaiveLookup.prototype.get = function(v) {
+      for (var i = 0; i < this.objs.length; i++) {
+        if (this.objs[i] === v) return i;
+      }
+    };
+    NaiveLookup.prototype.set = function() {
+      // Do nothing here. [MlObjectTable.store] will push to [this.objs] directly.
+    };
+
+    return function MlObjectTable() {
+      this.objs = []; this.lookup = new NaiveLookup(this.objs);
+    };
+  }();
+}
+else {
+  MlObjectTable = function MlObjectTable() {
+    this.objs = []; this.lookup = new joo_global_object.WeakMap();
+  };
+}
+
+MlObjectTable.prototype.store = function(v) {
+  this.lookup.set(v, this.objs.length);
+  this.objs.push(v);
+}
+
+MlObjectTable.prototype.recall = function(v) {
+  var i = this.lookup.get(v);
+  return (i === undefined)
+    ? undefined : this.objs.length - i;   /* index is relative */
+}
+
+//Provides: caml_legacy_custom_code
+//Version: >= 4.08
+var caml_legacy_custom_code = false
+
+//Provides: caml_legacy_custom_code
+//Version: < 4.08
+var caml_legacy_custom_code = true
+
 //Provides: caml_output_val
 //Requires: caml_int64_to_bytes, caml_failwith
 //Requires: caml_int64_bits_of_float
-//Requires: MlString, caml_ml_string_length, caml_string_unsafe_get
+//Requires: MlBytes, caml_ml_string_length, caml_string_unsafe_get
+//Requires: MlObjectTable, caml_list_to_js_array, caml_legacy_custom_code, caml_custom_ops
+//Requires: caml_invalid_argument
 var caml_output_val = function (){
   function Writer () { this.chunk = []; }
   Writer.prototype = {
@@ -295,11 +463,22 @@ var caml_output_val = function (){
       for (var i = size - 8;i >= 0;i -= 8)
         this.chunk[this.chunk_idx++] = (value >> i) & 0xFF;
     },
+    write_at:function (pos, size, value) {
+      var pos = pos;
+      for (var i = size - 8;i >= 0;i -= 8)
+        this.chunk[pos++] = (value >> i) & 0xFF;
+    },
     write_code:function (size, code, value) {
       this.chunk[this.chunk_idx++] = code;
       for (var i = size - 8;i >= 0;i -= 8)
         this.chunk[this.chunk_idx++] = (value >> i) & 0xFF;
     },
+    write_shared:function (offset) {
+      if (offset < (1 << 8)) this.write_code(8, 0x04 /*cst.CODE_SHARED8*/, offset);
+      else if (offset < (1 << 16)) this.write_code(16, 0x05 /*cst.CODE_SHARED16*/, offset);
+      else this.write_code(32, 0x06 /*cst.CODE_SHARED32*/, offset);
+    },
+    pos:function () { return this.chunk_idx },
     finalize:function () {
       this.block_len = this.chunk_idx - 20;
       this.chunk_idx = 0;
@@ -311,24 +490,72 @@ var caml_output_val = function (){
       return this.chunk;
     }
   }
-  return function (v) {
+  return function (v, flags) {
+    flags = caml_list_to_js_array(flags);
+
+    var no_sharing = (flags.indexOf(0 /*Marshal.No_sharing*/) !== -1),
+        closures =  (flags.indexOf(1 /*Marshal.Closures*/) !== -1);
+        /* Marshal.Compat_32 is redundant since integers are 32-bit anyway */
+
+    if (closures)
+      joo_global_object.console.warn("in caml_output_val: flag Marshal.Closures is not supported.");
+
     var writer = new Writer ();
     var stack = [];
+    var intern_obj_table = no_sharing ? null : new MlObjectTable();
+
+    function memo(v) {
+      if (no_sharing) return false;
+      var existing_offset = intern_obj_table.recall(v);
+      if (existing_offset) { writer.write_shared(existing_offset); return true; }
+      else { intern_obj_table.store(v); return false; }
+    }
+
     function extern_rec (v) {
-      if (v instanceof Array && v[0] === (v[0]|0)) {
-        if (v[0] == 255) {
-          // Int64
+      if (v.caml_custom || (v instanceof Array && v[0] === 255)) {
+        if (memo(v)) return;
+        var name = v.caml_custom || "_j";
+        var ops = caml_custom_ops[name];
+        var sz_32_64 = [0,0];
+        if(!ops.serialize)
+          caml_invalid_argument("output_value: abstract value (Custom)");
+        if(caml_legacy_custom_code) {
           writer.write (8, 0x12 /*cst.CODE_CUSTOM*/);
-          for (var i = 0; i < 3; i++) writer.write (8, "_j\0".charCodeAt(i));
-          var b = caml_int64_to_bytes (v);
-          for (var i = 0; i < 8; i++) writer.write (8, b[i]);
-          writer.size_32 += 4;
-          writer.size_64 += 3;
-          return;
+          for (var i = 0; i < name.length; i++)
+            writer.write (8, name.charCodeAt(i));
+          writer.write(8, 0);
+          ops.serialize(writer, v, sz_32_64);
+        } else if(ops.fixed_length == undefined){
+          writer.write (8, 0x18 /*cst.CODE_CUSTOM_LEN*/);
+          for (var i = 0; i < name.length; i++)
+            writer.write (8, name.charCodeAt(i));
+          writer.write(8, 0);
+          var header_pos = writer.pos ();
+          for(var i = 0; i < 12; i++) {
+            writer.write(8, 0);
+          }
+          ops.serialize(writer, v, sz_32_64);
+          writer.write_at(header_pos, 32, sz_32_64[0]);
+          writer.write_at(header_pos + 4, 32, 0); // zero
+          writer.write_at(header_pos + 8, 32, sz_32_64[1]);
+        } else {
+          writer.write (8, 0x19 /*cst.CODE_CUSTOM_FIXED*/);
+          for (var i = 0; i < name.length; i++)
+            writer.write (8, name.charCodeAt(i));
+          writer.write(8, 0);
+          var old_pos = writer.pos();
+          ops.serialize(writer, v, sz_32_64);
+          if (ops.fixed_length != writer.pos() - old_pos)
+            caml_failwith("output_value: incorrect fixed sizes specified by " + name);
         }
+        writer.size_32 += 2 + ((sz_32_64[0] + 3) >> 2);
+        writer.size_64 += 2 + ((sz_32_64[1] + 7) >> 3);
+      }
+      else if (v instanceof Array && v[0] === (v[0]|0)) {
         if (v[0] == 251) {
           caml_failwith("output_value: abstract value (Abstract)");
         }
+        if (v.length > 1 && memo(v)) return;
         if (v[0] < 16 && v.length - 1 < 8)
           writer.write (8, 0x80 /*cst.PREFIX_SMALL_BLOCK*/ + v[0] + ((v.length - 1)<<4));
         else
@@ -336,7 +563,8 @@ var caml_output_val = function (){
         writer.size_32 += v.length;
         writer.size_64 += v.length;
         if (v.length > 1) stack.push (v, 1);
-      } else if (v instanceof MlString) {
+      } else if (v instanceof MlBytes) {
+        if (memo(v)) return;
         var len = caml_ml_string_length(v);
         if (len < 0x20)
           writer.write (8, 0x20 /*cst.PREFIX_SMALL_STRING*/ + len);
@@ -351,18 +579,18 @@ var caml_output_val = function (){
       } else {
         if (v != (v|0)){
           var type_of_v = typeof v;
-//
-// If a float happens to be an integer it is serialized as an integer
-// (Js_of_ocaml cannot tell whether the type of an integer number is
-// float or integer.) This can result in unexpected crashes when
-// unmarshalling using the standard runtime. It seems better to
-// systematically fail on marshalling.
-//
-//          if(type_of_v != "number")
+          //
+          // If a float happens to be an integer it is serialized as an integer
+          // (Js_of_ocaml cannot tell whether the type of an integer number is
+          // float or integer.) This can result in unexpected crashes when
+          // unmarshalling using the standard runtime. It seems better to
+          // systematically fail on marshalling.
+          //
+          //          if(type_of_v != "number")
           caml_failwith("output_value: abstract value ("+type_of_v+")");
-//          var t = caml_int64_to_bytes(caml_int64_bits_of_float(v));
-//          writer.write (8, 0x0B /*cst.CODE_DOUBLE_BIG*/);
-//          for(var i = 0; i<8; i++){writer.write(8,t[i])}
+          //          var t = caml_int64_to_bytes(caml_int64_bits_of_float(v));
+          //          writer.write (8, 0x0B /*cst.CODE_DOUBLE_BIG*/);
+          //          for(var i = 0; i<8; i++){writer.write(8,t[i])}
         }
         else if (v >= 0 && v < 0x40) {
           writer.write (8, 0X40 /*cst.PREFIX_SMALL_INT*/ + v);
@@ -383,24 +611,29 @@ var caml_output_val = function (){
       if (i + 1 < v.length) stack.push (v, i + 1);
       extern_rec (v[i]);
     }
-    writer.finalize ();
+    if (intern_obj_table) writer.obj_counter = intern_obj_table.objs.length;
+    writer.finalize();
     return writer.chunk;
   }
 } ();
 
 //Provides: caml_output_value_to_string mutable
 //Requires: caml_output_val, caml_string_of_array
-function caml_output_value_to_string (v, _fl) {
-  /* ignores flags... */
-  return caml_string_of_array (caml_output_val (v));
+function caml_output_value_to_string (v, flags) {
+  return caml_string_of_array (caml_output_val (v, flags));
+}
+
+//Provides: caml_output_value_to_bytes mutable
+//Requires: caml_output_val, caml_string_of_array
+function caml_output_value_to_bytes (v, flags) {
+  return caml_string_of_array (caml_output_val (v, flags));
 }
 
 //Provides: caml_output_value_to_buffer
-//Requires: caml_output_val, caml_failwith, caml_blit_string
-function caml_output_value_to_buffer (s, ofs, len, v, _fl) {
-  /* ignores flags... */
-  var t = caml_output_val (v);
+//Requires: caml_output_val, caml_failwith, caml_blit_bytes
+function caml_output_value_to_buffer (s, ofs, len, v, flags) {
+  var t = caml_output_val (v, flags);
   if (t.length > len) caml_failwith ("Marshal.to_buffer: buffer overflow");
-  caml_blit_string(t, 0, s, ofs, t.length);
+  caml_blit_bytes(t, 0, s, ofs, t.length);
   return 0;
 }
