@@ -48,7 +48,8 @@
 
 //Provides: caml_str_repeat
 function caml_str_repeat(n, s) {
-  if (s.repeat) return s.repeat(n); // ECMAscript 6 and Firefox 24+
+  if(n == 0) return "";
+  if (s.repeat) {return s.repeat(n);} // ECMAscript 6 and Firefox 24+
   var r = "", l = 0;
   if (n == 0) return r;
   for(;;) {
@@ -67,6 +68,10 @@ function caml_str_repeat(n, s) {
 
 //Provides: caml_subarray_to_string
 //Requires: raw_array_sub
+//Weakdef
+// Pre ECMAScript 5, [apply] would not support array-like object.
+// In such setup, Typed_array would be implemented as polyfill, and [f.apply] would
+// fail here. Mark the primitive as Weakdef, so that people can override it easily.
 function caml_subarray_to_string (a, i, len) {
   var f = String.fromCharCode;
   if (i == 0 && len <= 4096 && len == a.length) return f.apply (null, a);
@@ -136,12 +141,12 @@ function caml_utf16_of_utf8(s) {
             v = c - 0xe2080;
             if ((v < 0x800) || ((v >= 0xd7ff) && (v < 0xe000))) v = 2;
           } else {
-              v = 3;
-              if ((++i < l) && (((c2 = s.charCodeAt(i)) & -64) == 128) &&
-                  (c1 < 0xf5)) {
-                v = c2 - 0x3c82080 + (c << 6);
-                if (v < 0x10000 || v > 0x10ffff) v = 3;
-              }
+            v = 3;
+            if ((++i < l) && (((c2 = s.charCodeAt(i)) & -64) == 128) &&
+                (c1 < 0xf5)) {
+              v = c2 - 0x3c82080 + (c << 6);
+              if (v < 0x10000 || v > 0x10ffff) v = 3;
+            }
           }
         }
       }
@@ -200,9 +205,21 @@ function caml_string_unsafe_get (s, i) {
   }
 }
 
-//Provides: caml_string_unsafe_set
+//Provides: caml_bytes_unsafe_get mutable
+function caml_bytes_unsafe_get (s, i) {
+  switch (s.t & 6) {
+  default: /* PARTIAL */
+    if (i >= s.c.length) return 0;
+  case 0: /* BYTES */
+    return s.c.charCodeAt(i);
+  case 4: /* ARRAY */
+    return s.c[i]
+  }
+}
+
+//Provides: caml_bytes_unsafe_set
 //Requires: caml_convert_string_to_array
-function caml_string_unsafe_set (s, i, c) {
+function caml_bytes_unsafe_set (s, i, c) {
   // The OCaml compiler uses Char.unsafe_chr on integers larger than 255!
   c &= 0xff;
   if (s.t != 4 /* ARRAY */) {
@@ -217,32 +234,176 @@ function caml_string_unsafe_set (s, i, c) {
   return 0;
 }
 
+//Provides: caml_string_unsafe_set
+//Requires: caml_bytes_unsafe_set
+function caml_string_unsafe_set (s, i, c) {
+  return caml_bytes_unsafe_set(s,i,c);
+}
+
 //Provides: caml_string_bound_error
 //Requires: caml_invalid_argument
 function caml_string_bound_error () {
   caml_invalid_argument ("index out of bounds");
 }
 
-//Provides: caml_string_get mutable
+//Provides: caml_string_get
 //Requires: caml_string_bound_error, caml_string_unsafe_get
 function caml_string_get (s, i) {
   if (i >>> 0 >= s.l) caml_string_bound_error();
   return caml_string_unsafe_get (s, i);
 }
 
+//Provides: caml_string_get16
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+function caml_string_get16(s,i) {
+  if (i >>> 0 >= s.l - 1) caml_string_bound_error();
+  var b1 = caml_string_unsafe_get (s, i),
+      b2 = caml_string_unsafe_get (s, i + 1);
+  return (b2 << 8 | b1);
+}
+
+//Provides: caml_bytes_get16
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+function caml_bytes_get16(s,i) {
+  if (i >>> 0 >= s.l - 1) caml_string_bound_error();
+  var b1 = caml_string_unsafe_get (s, i),
+      b2 = caml_string_unsafe_get (s, i + 1);
+  return (b2 << 8 | b1);
+}
+
+//Provides: caml_string_get32
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+function caml_string_get32(s,i) {
+  if (i >>> 0 >= s.l - 3) caml_string_bound_error();
+  var b1 = caml_string_unsafe_get (s, i),
+      b2 = caml_string_unsafe_get (s, i + 1),
+      b3 = caml_string_unsafe_get (s, i + 2),
+      b4 = caml_string_unsafe_get (s, i + 3);
+  return (b4 << 24 | b3 << 16 | b2 << 8 | b1);
+}
+
+//Provides: caml_bytes_get32
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+function caml_bytes_get32(s,i) {
+  if (i >>> 0 >= s.l - 3) caml_string_bound_error();
+  var b1 = caml_string_unsafe_get (s, i),
+      b2 = caml_string_unsafe_get (s, i + 1),
+      b3 = caml_string_unsafe_get (s, i + 2),
+      b4 = caml_string_unsafe_get (s, i + 3);
+  return (b4 << 24 | b3 << 16 | b2 << 8 | b1);
+}
+
+//Provides: caml_string_get64
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+//Requires: caml_int64_of_bytes
+function caml_string_get64(s,i) {
+  if (i >>> 0 >= s.l - 7) caml_string_bound_error();
+  var a = new Array(8);
+  for(var j = 0; j < 8; j++){
+    a[7 - j] = caml_string_unsafe_get (s, i + j);
+  }
+  return caml_int64_of_bytes(a);
+}
+
+//Provides: caml_bytes_get64
+//Requires: caml_string_unsafe_get, caml_string_bound_error
+//Requires: caml_int64_of_bytes
+function caml_bytes_get64(s,i) {
+  if (i >>> 0 >= s.l - 7) caml_string_bound_error();
+  var a = new Array(8);
+  for(var j = 0; j < 8; j++){
+    a[7 - j] = caml_string_unsafe_get (s, i + j);
+  }
+  return caml_int64_of_bytes(a);
+}
+
+//Provides: caml_bytes_get
+//Requires: caml_string_bound_error, caml_bytes_unsafe_get
+function caml_bytes_get (s, i) {
+  if (i >>> 0 >= s.l) caml_string_bound_error();
+  return caml_bytes_unsafe_get (s, i);
+}
+
 //Provides: caml_string_set
-//Requires: caml_string_bound_error, caml_string_unsafe_set
+//Requires: caml_string_unsafe_set, caml_string_bound_error
 function caml_string_set (s, i, c) {
   if (i >>> 0 >= s.l) caml_string_bound_error();
   return caml_string_unsafe_set (s, i, c);
 }
 
-//Provides: MlString
+//Provides: caml_bytes_set16
+//Requires: caml_string_bound_error, caml_string_unsafe_set
+function caml_bytes_set16(s,i,i16){
+  if (i >>> 0 >= s.l - 1) caml_string_bound_error();
+  var b2 = 0xFF & i16 >> 8,
+      b1 = 0xFF & i16;
+  caml_string_unsafe_set (s, i + 0, b1);
+  caml_string_unsafe_set (s, i + 1, b2);
+  return 0
+}
+
+//Provides: caml_string_set16
+//Requires: caml_bytes_set16
+function caml_string_set16(s,i,i16){
+  return caml_bytes_set16(s,i,i16);
+}
+
+//Provides: caml_bytes_set32
+//Requires: caml_string_bound_error, caml_string_unsafe_set
+function caml_bytes_set32(s,i,i32){
+  if (i >>> 0 >= s.l - 3) caml_string_bound_error();
+  var b4 = 0xFF & i32 >> 24,
+      b3 = 0xFF & i32 >> 16,
+      b2 = 0xFF & i32 >> 8,
+      b1 = 0xFF & i32;
+  caml_string_unsafe_set (s, i + 0, b1);
+  caml_string_unsafe_set (s, i + 1, b2);
+  caml_string_unsafe_set (s, i + 2, b3);
+  caml_string_unsafe_set (s, i + 3, b4);
+  return 0
+}
+
+//Provides: caml_string_set32
+//Requires: caml_bytes_set32
+function caml_string_set32(s,i,i32){
+  return caml_bytes_set32(s,i,i32);
+}
+
+//Provides: caml_bytes_set64
+//Requires: caml_string_bound_error, caml_string_unsafe_set
+//Requires: caml_int64_to_bytes
+function caml_bytes_set64(s,i,i64){
+  if (i >>> 0 >= s.l - 7) caml_string_bound_error();
+  var a = caml_int64_to_bytes(i64);
+  for(var j = 0; j < 8; j++) {
+    caml_string_unsafe_set (s, i + 7 - j, a[j]);
+  }
+  return 0
+}
+
+//Provides: caml_string_set64
+//Requires: caml_bytes_set64
+function caml_string_set64(s,i,i64){
+  return caml_bytes_set64(s,i,i64);
+}
+
+//Provides: caml_bytes_set
+//Requires: caml_string_bound_error, caml_bytes_unsafe_set
+function caml_bytes_set (s, i, c) {
+  if (i >>> 0 >= s.l) caml_string_bound_error();
+  return caml_bytes_unsafe_set (s, i, c);
+}
+
+//Provides: MlBytes
 //Requires: caml_to_js_string
-function MlString (tag, contents, length) {
+function MlBytes (tag, contents, length) {
   this.t=tag; this.c=contents; this.l=length;
 }
-MlString.prototype = { toString:function(){return caml_to_js_string(this)} }
+MlBytes.prototype.toString = function(){return caml_to_js_string(this)};
+MlBytes.prototype.slice = function (){
+  var content = this.t == 4 ? this.c.slice() : this.c;
+  return new MlBytes(this.t,content,this.l);
+}
 
 //Provides: caml_convert_string_to_bytes
 //Requires: caml_str_repeat, caml_subarray_to_string
@@ -258,7 +419,12 @@ function caml_convert_string_to_bytes (s) {
 //Provides: caml_convert_string_to_array
 function caml_convert_string_to_array (s) {
   /* Assumes not ARRAY */
-  var a = new Array(s.l), b = s.c, l = b.length, i = 0;
+  if(joo_global_object.Uint8Array) {
+    var a = new joo_global_object.Uint8Array(s.l);
+  } else {
+    var a = new Array(s.l);
+  }
+  var b = s.c, l = b.length, i = 0;
   for (; i < l; i++) a[i] = b.charCodeAt(i);
   for (l = s.l; i < l; i++) a[i] = 0;
   s.c = a;
@@ -273,34 +439,42 @@ function caml_array_of_string (s) {
   return s.c;
 }
 
-//Provides: caml_bytes_of_string mutable
+//Provides: caml_jsbytes_of_string mutable
 //Requires: caml_convert_string_to_bytes
-function caml_bytes_of_string (s) {
+function caml_jsbytes_of_string (s) {
   if ((s.t & 6) != 0 /* BYTES */) caml_convert_string_to_bytes(s);
   return s.c;
 }
 
 //Provides: caml_js_to_string const
-//Requires: caml_is_ascii, caml_utf8_of_utf16, MlString
+//Requires: caml_is_ascii, caml_utf8_of_utf16, MlBytes
 function caml_js_to_string (s) {
   var tag = 9 /* BYTES | ASCII */;
   if (!caml_is_ascii(s))
     tag = 8 /* BYTES | NOT_ASCII */, s = caml_utf8_of_utf16(s);
-  return new MlString(tag, s, s.length);
+  return new MlBytes(tag, s, s.length);
 }
 
 //Provides: caml_create_string const
-//Requires: MlString,caml_invalid_argument
+//Requires: MlBytes,caml_invalid_argument
 function caml_create_string(len) {
   if (len < 0) caml_invalid_argument("String.create");
-  return new MlString(len?2:9,"",len);
+  return new MlBytes(len?2:9,"",len);
 }
-//Provides: caml_new_string
-//Requires: MlString
-function caml_new_string (s) { return new MlString(0,s,s.length); }
+//Provides: caml_create_bytes const
+//Requires: MlBytes,caml_invalid_argument
+function caml_create_bytes(len) {
+  if (len < 0) caml_invalid_argument("Bytes.create");
+  return new MlBytes(len?2:9,"",len);
+}
+
+//Provides: caml_new_string const (const)
+//Requires: MlBytes
+function caml_new_string (s) { return new MlBytes(0,s,s.length); }
+
 //Provides: caml_string_of_array
-//Requires: MlString
-function caml_string_of_array (a) { return new MlString(4,a,a.length); }
+//Requires: MlBytes
+function caml_string_of_array (a) { return new MlBytes(4,a,a.length); }
 
 //Provides: caml_string_compare mutable
 //Requires: caml_convert_string_to_bytes
@@ -310,18 +484,41 @@ function caml_string_compare(s1, s2) {
   return (s1.c < s2.c)?-1:(s1.c > s2.c)?1:0;
 }
 
-//Provides: caml_string_equal mutable
+
+//Provides: caml_bytes_compare mutable
+//Requires: caml_convert_string_to_bytes
+function caml_bytes_compare(s1, s2) {
+  (s1.t & 6) && caml_convert_string_to_bytes(s1);
+  (s2.t & 6) && caml_convert_string_to_bytes(s2);
+  return (s1.c < s2.c)?-1:(s1.c > s2.c)?1:0;
+}
+
+//Provides: caml_string_equal mutable (const, const)
 //Requires: caml_convert_string_to_bytes
 function caml_string_equal(s1, s2) {
+  if(s1 === s2) return 1;
   (s1.t & 6) && caml_convert_string_to_bytes(s1);
   (s2.t & 6) && caml_convert_string_to_bytes(s2);
   return (s1.c == s2.c)?1:0;
 }
 
-//Provides: caml_string_notequal mutable
+//Provides: caml_bytes_equal mutable (const, const)
+//Requires: caml_convert_string_to_bytes
+function caml_bytes_equal(s1, s2) {
+  if(s1 === s2) return 1;
+  (s1.t & 6) && caml_convert_string_to_bytes(s1);
+  (s2.t & 6) && caml_convert_string_to_bytes(s2);
+  return (s1.c == s2.c)?1:0;
+}
+
+//Provides: caml_string_notequal mutable (const, const)
 //Requires: caml_string_equal
 function caml_string_notequal(s1, s2) { return 1-caml_string_equal(s1, s2); }
 
+//Provides: caml_bytes_notequal mutable (const, const)
+//Requires: caml_string_equal
+function caml_bytes_notequal(s1, s2) { return 1-caml_string_equal(s1, s2); }
+
 //Provides: caml_string_lessequal mutable
 //Requires: caml_convert_string_to_bytes
 function caml_string_lessequal(s1, s2) {
@@ -330,6 +527,14 @@ function caml_string_lessequal(s1, s2) {
   return (s1.c <= s2.c)?1:0;
 }
 
+//Provides: caml_bytes_lessequal mutable
+//Requires: caml_convert_string_to_bytes
+function caml_bytes_lessequal(s1, s2) {
+  (s1.t & 6) && caml_convert_string_to_bytes(s1);
+  (s2.t & 6) && caml_convert_string_to_bytes(s2);
+  return (s1.c <= s2.c)?1:0;
+}
+
 //Provides: caml_string_lessthan mutable
 //Requires: caml_convert_string_to_bytes
 function caml_string_lessthan(s1, s2) {
@@ -338,11 +543,24 @@ function caml_string_lessthan(s1, s2) {
   return (s1.c < s2.c)?1:0;
 }
 
+//Provides: caml_bytes_lessthan mutable
+//Requires: caml_convert_string_to_bytes
+function caml_bytes_lessthan(s1, s2) {
+  (s1.t & 6) && caml_convert_string_to_bytes(s1);
+  (s2.t & 6) && caml_convert_string_to_bytes(s2);
+  return (s1.c < s2.c)?1:0;
+}
+
 //Provides: caml_string_greaterequal
 //Requires: caml_string_lessequal
 function caml_string_greaterequal(s1, s2) {
   return caml_string_lessequal(s2,s1);
 }
+//Provides: caml_bytes_greaterequal
+//Requires: caml_bytes_lessequal
+function caml_bytes_greaterequal(s1, s2) {
+  return caml_bytes_lessequal(s2,s1);
+}
 
 //Provides: caml_string_greaterthan
 //Requires: caml_string_lessthan
@@ -350,9 +568,15 @@ function caml_string_greaterthan(s1, s2) {
   return caml_string_lessthan(s2, s1);
 }
 
-//Provides: caml_fill_string
+//Provides: caml_bytes_greaterthan
+//Requires: caml_bytes_lessthan
+function caml_bytes_greaterthan(s1, s2) {
+  return caml_bytes_lessthan(s2, s1);
+}
+
+//Provides: caml_fill_bytes
 //Requires: caml_str_repeat, caml_convert_string_to_array
-function caml_fill_string(s, i, l, c) {
+function caml_fill_bytes(s, i, l, c) {
   if (l > 0) {
     if (i == 0 && (l >= s.l || (s.t == 2 /* PARTIAL */ && l >= s.c.length))) {
       if (c == 0) {
@@ -370,27 +594,35 @@ function caml_fill_string(s, i, l, c) {
   return 0;
 }
 
-//Provides: caml_blit_string
+//Provides: caml_fill_string
+//Requires: caml_fill_bytes
+var caml_fill_string = caml_fill_bytes
+
+//Provides: caml_blit_bytes
 //Requires: caml_subarray_to_string, caml_convert_string_to_array
-function caml_blit_string(s1, i1, s2, i2, len) {
+function caml_blit_bytes(s1, i1, s2, i2, len) {
   if (len == 0) return 0;
   if ((i2 == 0) &&
       (len >= s2.l || (s2.t == 2 /* PARTIAL */ && len >= s2.c.length))) {
     s2.c = (s1.t == 4 /* ARRAY */)?
-             caml_subarray_to_string(s1.c, i1, len):
-             (i1 == 0 && s1.c.length == len)?s1.c:s1.c.substr(i1, len);
+      caml_subarray_to_string(s1.c, i1, len):
+      (i1 == 0 && s1.c.length == len)?s1.c:s1.c.substr(i1, len);
     s2.t = (s2.c.length == s2.l)?0 /* BYTES | UNKOWN */ :2; /* PARTIAL */
   } else if (s2.t == 2 /* PARTIAL */ && i2 == s2.c.length) {
     s2.c += (s1.t == 4 /* ARRAY */)?
-             caml_subarray_to_string(s1.c, i1, len):
-             (i1 == 0 && s1.c.length == len)?s1.c:s1.c.substr(i1, len);
+      caml_subarray_to_string(s1.c, i1, len):
+      (i1 == 0 && s1.c.length == len)?s1.c:s1.c.substr(i1, len);
     s2.t = (s2.c.length == s2.l)?0 /* BYTES | UNKOWN */ :2; /* PARTIAL */
   } else {
     if (s2.t != 4 /* ARRAY */) caml_convert_string_to_array(s2);
     var c1 = s1.c, c2 = s2.c;
-    if (s1.t == 4 /* ARRAY */)
-      for (var i = 0; i < len; i++) c2 [i2 + i] = c1 [i1 + i];
-    else {
+    if (s1.t == 4 /* ARRAY */) {
+      if (i2 <= i1) {
+        for (var i = 0; i < len; i++) c2 [i2 + i] = c1 [i1 + i];
+      } else {
+        for (var i = len - 1; i >= 0; i--) c2 [i2 + i] = c1 [i1 + i];
+      }
+    } else {
       var l = Math.min (len, c1.length - i1);
       for (var i = 0; i < l; i++) c2 [i2 + i] = c1.charCodeAt(i1 + i);
       for (; i < len; i++) c2 [i2 + i] = 0;
@@ -399,5 +631,21 @@ function caml_blit_string(s1, i1, s2, i2, len) {
   return 0;
 }
 
+//Provides: caml_blit_string
+//Requires: caml_blit_bytes
+function caml_blit_string(s1, i1, s2, i2, len) {
+  // TODO: s1 -> string to bytes
+  return caml_blit_bytes(s1, i1, s2, i2, len);
+}
+
 //Provides: caml_ml_string_length const
 function caml_ml_string_length(s) { return s.l }
+
+//Provides: caml_ml_bytes_length const
+function caml_ml_bytes_length(s) { return s.l }
+
+//Provides: caml_string_of_bytes const
+function caml_string_of_bytes(s) { return s}
+
+//Provides: caml_bytes_of_string const
+function caml_bytes_of_string(s) { return s}
